<?php

namespace EventPortal\Domain\Approval\Models;

use EventPortal\Concerns\AddToFillable;
use EventPortal\Domain\Comment\Models\Comment;
use EventPortal\Domain\Event\Models\Event;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\MorphMany;
use Illuminate\Database\Eloquent\Relations\Pivot;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class ApprovalData extends Pivot implements AuditableInterface
{
    use Auditable;
    use AddToFillable;

    /** @var string */
    protected $table = 'approval_data';

    /** @var array<string> */
    protected $fillable = [
        'event_id',
        'approval_id',
        'key',
        'value',
    ];

    public $incrementing = true;

    public function comments(): MorphMany
    {
        return $this->morphMany(Comment::class, 'commentable');
    }

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class);
    }

    public function transitionRouteName(): string
    {
        return 'admin.event-approval.transition';
    }

    public function transitionRouteParameters(): array
    {
        return [
            'event' => $this->id,
        ];
    }

    protected function getGraph(): string
    {
        return 'eventApproval';
    }
}
