<?php

namespace EventPortal\Domain\Delegate\Models;

use EventPortal\Concerns\AddToCasts;
use EventPortal\Concerns\AddToFillable;
use EventPortal\Concerns\HasAddress;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Model;
use EventPortal\Domain\Table\Concerns\HasTable;
use EventPortal\Domain\Util\Notification\Models\EventNotification;
use EventPortal\Domain\Util\Status;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasManyThrough;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class Delegate extends Model implements AuditableInterface
{
    use HasTable, AddToFillable, AddToCasts, Auditable, HasAddress;

    const KEY = 'delegate';

    /** @var array<string> */
    public $with = [
        'user',
    ];

    /** @var array<string> */
    protected $fillable = [
        'user_id',
        'pa_email',
    ];

    public function events(): BelongsToMany
    {
        return $this
            ->belongsToMany(Event::class)
            ->using(EventDelegate::class)
            ->withPivot(['id', 'status'])
            ->withTimestamps();
    }

    public function user(): BelongsTo
    {
        return $this->belongsTo(config('auth.providers.users.model'), 'user_id');
    }

    public function notifications(): HasMany
    {
        return $this->hasMany(EventNotification::class);
    }

    public function formData(): HasMany
    {
        return $this->hasMany(
            DelegateFormData::class,
            'delegate_id',
            'id',
        );
    }

    public function formSteps(): HasManyThrough
    {
        return $this->hasManyThrough(
            FormStep::class,
            DelegateFormStep::class,
            'delegate_id',
            'id',
            'id',
            'form_step_id'
        )
            ->orderBy('delegate_form_step.updated_at', 'desc');
    }

    public function getReadableStatusAttribute(): string
    {
        return Status::make(EventDelegate::KEY)->getReadable($this->attributes['status']);
    }

    public function getNameAttribute(): string
    {
        return $this->user->first_name . ' ' . $this->user->last_name;
    }

    public function hasPA(): bool
    {
        return $this->pa_email !== null;
    }
}
