<?php

namespace EventPortal\Domain\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\ValueObjects\StepData;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;

class UpdateSteps
{
    public function execute(Collection $steps, Form $form): void
    {
        $this->deleteUnusedSteps($steps, $form);
        $steps->each(static function (StepData $step) use ($form): void {
            $formStep = FormStep::updateOrCreate(
                ['id' => $step->id],
                Arr::add($step->toArray(), 'form_id', $form->id)
            );

            (new UpdateSections)->execute($step->sections, $formStep);
        });
    }

    private function deleteUnusedSteps(Collection $steps, Form $form): void
    {
        FormStep::query()
            ->whereNotIn('id', $steps->pluck('id'))
            ->forForm($form)
            ->delete();
    }
}
