<?php

namespace EventPortal\Domain\Event\Form\Models;

use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class FormStepSection extends Model implements AuditableInterface
{
    use Auditable;

    const KEY = 'form_step_sections';

    /** @var array<string> */
    protected $fillable = [
        'name',
        'order',
        'form_step_id',
        'uuid',
    ];

    public function step(): BelongsTo
    {
        return $this->belongsTo(FormStep::class);
    }

    public function questions(): HasMany
    {
        return $this->hasMany(Question::class)->orderBy('order');
    }

    public function scopeForFormStep(Builder $query, FormStep $formStep): Builder
    {
        return $query->where('form_step_id', '=', $formStep->id);
    }

    public function asComponent(bool $forDisplay = false): Section
    {
        $component = new Section($this->name);

        $component->fields($this->questions->map->asComponent($forDisplay)->toArray());

        return $component;
    }
}
